//Datei: DS1821.C
// Hauptprogramm
//
// Przisionsthermometer
// mit PIC16F84 und DS1821
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// Copyright 2004
// 22.04.2004
// Compiler CC5x

// Verwendete LCD-Anzeige 2x16 Zeichen oder 2x8 Zeichen

#include "prozess.h"      /* Der verwendete Prozessor */
#include "ds1821.h"
#include <math16.h>

// Falls die Sensoren mal weiter auseinanderliegen kann
// man hier einen festen Offset eintragen.
// Bei +0.23 Abweichung trgt man +23 ein.
// Bei -0.23 Abweichung trgt man -23 ein.
// Wenn beide von der tatschlichen Temperatur abweichen,
// dann eben fr beide einen Offset angeben.
// Dann das Programm neu bersetzen.
#define INTOFFSET	(int16)0 //Offset fr internen Fhler
#define EXTOFFSET	(int16)10 //Offset fr externen Fhler

 // Watchdog off, Power up Timer on, XT Oscillator, Code Protect off
#pragma config |= 0x3FF1 //Configuration Word

//Globale Flags und Rckgabevariablen
int8 sign;       //Vorzeichen und 7Bit Temperaturwert
int16 temp;     //Temperatur
int16 countrem; //9 Bit Register 1
int16 countdeg; //9 Bit Register 2
int16 tempbuf;  //bergabevariable fr Anzeige
int16 tempmin_int;  //Kleinste gemessene Temperatur intern
int16 tempmin_ext;  //Kleinste gemessene Temperatur extern
int16 tempmax_int;  //grte gemessene Temperatur intern
int16 tempmax_ext;  //grte gemessene Temperatur extern
unsigned char mode; //Aktueller Fhler

#include "delay.c"
#include "lcd.c"
#include "binbcd16.c"
#include "ds1821io.c"


/* Hauptprogramm */
void main()
{
 unsigned char i;
 
 OPTION=0b11010110;	 //Prescaler 1:128 und Intern Clock fr TMR0	
 
// Der Takt des PIC betrgt 4MHz
// Ein Maschinenzyklus damit 1us

 TRISA=0x0F; // PortA 	alles Eingnge auer RA4
 TRISB=0b01000000; // PortB 	alles Ausgnge auer PB6 

 PORTB=0x40;    //Definierten Zustand einstellen
 PORTA=0x1F;    //Pullup Ra4

 LCDInit(); //LCD Anzeige einstellen

 LCDCls();  //Anzeige lschen

 mode=0; //Externer Fhler
 MasterReset();
 SendByte(WRITESTATUS);
 SendByte(0x41); //One Shot Operation einstellen
 Delay1ms(255);  //Etwas warten

 mode=1; //Interner Fhler
 MasterReset();
 SendByte(WRITESTATUS);
 SendByte(0x41); //One Shot Operation einstellen
 Delay1ms(255);  //Etwas warten

 tempmin_int=(int16)12500;
 tempmax_int=(int16)-5500;
 
 tempmin_ext=(int16)12500;
 tempmax_ext=(int16)-5500;

 while(1) // Anfang Programm Endlosschleife
  {
   MasterReset();      //Messung starten
   SendByte(STARTCONV);
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??

  //Hier Status Polling fr Ende der Messung
   PollStatus();
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??
   
   MasterReset();
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??

   SendByte(READTEMP);
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??

   ReadByte(8);       //Temperatur lesen
   sign=(int8)(tempbuf&0x00FF); //Nur die unteren 8 Bits nehmen !
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??
   
   //Die beiden 9 Bit Register lesen
   MasterReset();
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??
   SendByte(READCOUNT);
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??
   ReadByte(9);       //Remain lesen
   countrem=tempbuf;
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??

   MasterReset();
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??
   SendByte(LOADREM); //Zweites Register laden
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??

   MasterReset();
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??
   SendByte(READCOUNT);
   Delay1ms(1); //Ohne dies delay geht es nicht !!! Warum ??
   ReadByte(9);       //Degree lesen
   countdeg=tempbuf;
   
//Zum testen der Anzeige:
//   sign=-54;   // Ergebnis = -54.09
//   sign=54;   // Ergebnis = +53.91
//   sign=0;     // Ergebnis = -0.09
//   countrem=(int16)300; 
//   countdeg=(int16)512;

   //signed integer aus Zweierkomplement machen wenn negativ
   if(sign<0)
    {
     sign=sign+255;     //Einerkomplement
     sign++;            //Zweierkomplement
    }

   temp=(int16)(sign); //int8 zu int16

   //Temperatur * 100  => -5500 bis 12500
   //Schiebt den Wert um zwei Dezimalstellen nach links
   //So spare ich mir unten eine Subtraktion und eine Addition
   //im Floating Point Format
   tempbuf=temp;
   for(i=0; i<99; i++) temp+=tempbuf; //Die einfache Art * 100 zu nehmen

//Wenn countdeg=0 mte durch 0 geteilt werden. Abfangen
   if(countdeg>(int16)0) //Schade jetzt mu ich rechnen
    {
     //countdeg-countrem in integer rechnen Ergebnis in countrem
     countrem=countdeg-countrem;
     //Das Ergebnis ist immer positiv

     //Ich mchte zwei Stellen hinter dem Komma haben
     //Das Ergebnis der Division wre aber kleiner 1
     //Um wieder FloatingPoint Operation zu vermeiden
     //nehme ich den Zhler * 100
     tempbuf=countrem;
     for(i=0; i<99; i++) countrem+=tempbuf; //Die einfache Art * 100 zu nehmen
     
     countrem=countrem/countdeg; //Und teilen
    } 
   else //countdeg war 0, dann 100 * 1/2 LSB(=0.5) addieren
    { countrem=(int16)50; } //1/2 LSB
   
   temp+=countrem; //Genaue Temperatur ausrechnen
   //LSB = 0.5 * 100 = 50
   //Temperatur - LSB ausrechnen
   temp-=(int16)50;

   if(mode==1)
    {
     temp+=INTOFFSET; //Evtuellen Offset dazurechnen um Fhler abzugleichen
     LCDPos(1,1); //Interne Temperatur
    }
   else
    {
     temp+=EXTOFFSET; //Evtuellen Offset dazurechnen um Fhler abzugleichen
     LCDPos(2,1); //Externe Temperatur
    }

   //Temperatur fnfstellig anzeigen
   if(MINMAX_INT==1 && MINMAX_EXT==1) //Sonst flackert die Anzeige wenn ein Minmax aktiv ist
    {
     tempbuf=temp;
     ShowDisplay();
    }
    
   if(mode==1)
    {
     if(tempmin_int>temp) tempmin_int=temp; //Neue Temperatur ist kleiner
     if(tempmax_int<temp) tempmax_int=temp; //Neue Temperatur ist grer
    }
   else
    {
     if(tempmin_ext>temp) tempmin_ext=temp; //Neue Temperatur ist kleiner
     if(tempmax_ext<temp) tempmax_ext=temp; //Neue Temperatur ist grer
    }    

   //Je nach Zustand am PORT Temperatur Min/Max intern oder extern anzeigen
   if(MINMAX_INT==0)
    {
     tempbuf=tempmin_int;
     LCDPos(1,1);
     ShowDisplay();
     tempbuf=tempmax_int;
     LCDPos(2,1);
     ShowDisplay();
    } 

   //Je nach Zustand am PORT Temperatur Min oder Max anzeigen
   if(MINMAX_EXT==0)
    {
     tempbuf=tempmin_ext;
     LCDPos(1,1);
     ShowDisplay();
     tempbuf=tempmax_ext;
     LCDPos(2,1);
     ShowDisplay();
    } 

   if(MINMAX_INT==0 && MINMAX_EXT==0) //Beide Min.Max zurcksetzen
    {
     tempmin_int=(int16)12500;
     tempmax_int=(int16)-5500;
     tempmin_ext=(int16)12500;
     tempmax_ext=(int16)-5500;
    } 

   if(mode==1) mode=0; //Fhler immer abwechselnd abfragen
   else mode=1;

  } // Ende Programm Endlosschleife
}

//************************************************
// Zeigt die 5 Dezimalstellen auf dem LCD Display 
//************************************************
void ShowDisplay()
{
 if(tempbuf<0) //Falls negativ, Absolutwert anzeigen
  {
   tempbuf=-tempbuf;
   LCDWriteByte(0x2D); //Vorzeichen - auf Display
  }
 else
  {
   LCDWriteByte(0x2B); //Vorzeichen + auf Display
  } 

 B1=(unsigned char)(tempbuf>>8); //Highbyte
 B0=(unsigned char)(tempbuf&0xFF); //Lowbyte

 BinToBCD16(); //unsigned int zu BCD

 LCDWriteByte(R0+0x30); //Zehntausender
 ShowBCD(R1); //Tausender Hunderter
 LCDWriteByte(0x2E); //"." anzeigen
 ShowBCD(R2); //Zehner Einer
 LCDWriteByte(0xDF); //"" Einheit anzeigen
}
